#!/usr/bin/python

import sys
import argparse

def parseLine(l):

    (ts, ch0, ch1, *x)  = l.split(", ")

    ts = int(float(ts) * 10**9)

    return (ts, int(ch0), int(ch1))

def parsePulses(filename):

    with open(filename, "r") as f:
        lines = [l for l in f]
        lines = lines[1:]

        ch0Ts = []
        ch1Ts = []

        prevCh0Ts = -1
        prevCh1Ts = -1

        for l in lines:
            (ts, ch0, ch1) = parseLine(l)

            if ch0 == 1:
                # Aggregate close timestamps as they represent the same pulse
                if prevCh0Ts == -1 or (ts - prevCh0Ts) > 9000 * 1000:
                    ch0Ts.append(ts)
                    prevCh0Ts = ts
            if ch1 == 1:
                if prevCh1Ts == -1 or (ts - prevCh1Ts) > 9000 * 1000:
                    ch1Ts.append(ts)
                    prevCh1Ts = ts

        # If there is an offset
        if abs(ch0Ts[0] - ch1Ts[0]) > abs(ch0Ts[0] - ch1Ts[1]):
            ch1Ts = ch1Ts[1:]
        elif abs(ch0Ts[0] - ch1Ts[0]) > abs(ch0Ts[1] - ch1Ts[0]):
            ch0Ts = ch0Ts[1:]

        deltas = []

        for i in range(min(len(ch0Ts), len(ch1Ts))):
            deltas.append(ch0Ts[i] - ch1Ts[i])

        max_d = max(deltas)

        return deltas
    print("IO error")
    exit()

def parseToggles(filename):

    with open(filename, "r") as f:
        lines = [l for l in f]
        lines = lines[3:]
        values = [ parseLine(l) for l in lines]
        values_c1 = []
        values_c2 = []

        # If both signals happenned at the same, they get merged at export
        # We don't want this, so here we undo this merge
        (pt, pc1, pc2) = values[0]
        i = 0
        while i < len(values):
            (t, c1, c2) = values[i]
            if c1 != pc1 and c2 != pc2:
                values.insert(i, (t, c1, c2))
                values_c1.append(t)
                values_c2.append(t)
                i += 1
            elif c1 != pc1:
                values_c1.append(t)
            elif c2 != pc2:
                values_c2.append(t)
            (pt, pc1, pc2) = values[i]
            i += 1

        periods_c1 = [values_c1[i] - values_c1[i-1] for i in range(1, len(values_c1))]
        periods_c2 = [values_c2[i] - values_c2[i-1] for i in range(1, len(values_c2))]

        values = [ t for (t, *x) in values]
        deltas = []

        start = 1
        if (values[1] - values[0]) > (values[2] - values[1]):
            start = 2

        for i in range(start, len(values), 2):

            if (i+1) < len(values):
                if (values[i] - values[i - 1]) > (values[i + 1] - values[i]):
                    print("Instruction missed around line {} ? ({} - {} > {} - {})".format(i, values[i] / 10**9, values[i - 1] / 10**9, values[i + 1] / 10**9, values[i] / 10**9))
                    exit()
            deltas.append(values[i] - values[i-1])

        return (deltas, periods_c1, periods_c2)
    print("IO error")
    exit()

def generateHistogram(deltas):

    deltas = [abs(int(d / 1000)) for d in deltas]

    min_d = min(deltas)
    max_d = max(deltas)
    avg_d = sum(deltas) / len(deltas)

    histogram = [0 for i in range(max_d+1)]

    for d in deltas:
        histogram[d] += 1

    for i,h in enumerate(histogram):
        print("{:06d} {:015d}".format(i,h))

    print("# Min: {}".format(min_d))
    print("# Max: {}".format(max_d))
    print("# Avg: {}".format(avg_d))


def parse_args():
    parser = argparse
    parser = argparse.ArgumentParser(description='Measure analysis')
    parser.add_argument('-t', nargs=1, required=False, help='import shuttle-a20 json measures')
    parser.add_argument('-p', nargs=1, required=False, help='import shuttle json measures')

    args = parser.parse_args()

    if args.t is not None:
        filename = args.t[0]
        (deltas, periods_c1, periods_c2) = parseToggles(filename)
        print("# Shuttle-A20 synchronization signal jitter histogram")
        generateHistogram(deltas)
        generateHistogram(periods_c1 + periods_c2)

    if args.p is not None:
        filename = args.p[0]
        print("# Shuttle synchronization histogram")
        deltas = parsePulses(filename)
        generateHistogram(deltas)


parse_args()
